#
#Add some funcionality to check stdout and stderr-output with test/unit.
#
#Please be aware:
#Normally you should not test the output of methods, but the result of methods.
#
require 'test/unit'
module Test::Unit

#Class to catch the std output and stderr
class Catch_IO	< String
	def write( cont )
		self	<< cont
	end
end	#MyIO

#
#Extend the class TestCase with additional methods/assertions
#
class TestCase
  #Catch stdout for a given procedure (Proc-Element).
  def catch_stdout( procedure )
    $stdout = output = Catch_IO.new #temporary copy
    procedure.call
    $stdout = STDOUT
    output
  end
  #
  #Actual must be a procedure.
  #Another object (like a method call) would be executed first, before stdout could be redirected.
  def assert_stdout( expected, actual, message = nil )
    if actual.respond_to?(:call)
      output = catch_stdout( actual )
      full_message = build_message(message, "<?> expected in stdout, but was\n<?>.\n", expected, output)
      assert_block( full_message ){ expected  == output }
    else
      raise 'no Proc-object given'
      full_message = build_message(message, "<?> expected in stdout, but no Proc-object is given to test.\n", expected )
      assert_block( full_message ){ false } #force error
    end
  end
  #
  #Catch the stdout of a given block,
  #This method looks better for me, 
  def assert_stdout_block( expected, message = nil )
    if block_given? 
      $stdout = output = Catch_IO.new #temporary copy
      yield
      $stdout = STDOUT
      full_message = build_message(message, "<?> expected in stdout, but was\n<?>.\n", expected, output)
      assert_block( full_message ){ expected  == output }
    else
      raise 'no block given'
      full_message = build_message(message, "<?> expected in stdout, but no block to test.\n", expected )
      assert_block( full_message ){ false }
    end
  end
  #Catch stderr for a given procedure (Proc-Element).
  def catch_stderr( procedure )
    $stderr = output = Catch_IO.new #temporary copy
    procedure.call
    $stderr = STDERR
    output
  end
  #
  #Actual must be a procedure.
  #Another object (like a method call) would be executed first, before stderr could be redirected.
  def assert_stderr( expected, actual, message = nil )
    if actual.respond_to?(:call)
      output = catch_stderr( actual )
      full_message = build_message(message, "<?> expected in stderr, but was\n<?>.\n", expected, output)
      assert_block( full_message ){ expected  == output }
    else
      raise 'no Proc-object given'
      full_message = build_message(message, "<?> expected in stderr, but no Proc-object is given to test.\n", expected )
      assert_block( full_message ){ false } #force error
    end
  end
  #
  #Catch the stderr of a given block,
  #This method looks better for me, 
  def assert_stderr_block( expected, message = nil )
    if block_given? 
      $stderr = output = Catch_IO.new #temporary copy
      yield
      $stderr = STDERR
      full_message = build_message(message, "<?> expected in stderr, but was\n<?>.\n", expected, output)
      assert_block( full_message ){ expected  == output }
    else
      raise 'no block given'
      full_message = build_message(message, "<?> expected in stderr, but no block to test.\n", expected )
      assert_block( full_message ){ false }
    end
  end
end   #class TestCase
end   #module Test::Unit

if $0 == __FILE__
class MyTest < Test::Unit::TestCase 
  def any_output()
    puts "Just some output"
  end
  def test_stdout_ok()
    assert_stdout("xx\n", proc{ puts 'xx' } )
    assert_stdout("Just some output\n", proc{ any_output()} )
    assert_stdout_block("xx\n"){ puts 'xx' }
    assert_stdout_block("Just some output\n"){ any_output() }
    assert_equal("xx\n", catch_stdout(proc{ puts 'xx' }) )
  end
  def test_stdout_err()
    #Error: No Proc-object
    assert_raise(RuntimeError){ assert_stdout("xx\n", 1+1) }
    #Error: No Block
    assert_raise(RuntimeError){ assert_stdout_block("xx\n") }
  end
  #Helper method to get content to stderr.
  def any_output2stderr()
    $stderr << "Just some output on stderr"
  end
  def test_stderr_ok()
    #~ assert_stderr("xx\n", proc{ puts 'xx' } )
    assert_stderr("Just some output on stderr", proc{ any_output2stderr()} )
    #~ assert_stderr_block("xx\n"){ puts 'xx' }
    assert_stderr_block("Just some output on stderr"){ any_output2stderr() }
    #~ assert_equal("xx\n", catch_stderr(proc{ puts 'xx' }) )
    assert_equal("Just some output on stderr", catch_stderr(proc{ any_output2stderr() } ) )
  end
  def test_stderr_err()
    #Error: No Proc-object
    assert_raise(RuntimeError){ assert_stderr("xx\n", 1+1) }
    #Error: No Block
    assert_raise(RuntimeError){ assert_stderr_block("xx\n") }
  end
end #class MyTest
end #$0 == __FILE__

__END__

class MyIO	< String
	def write( cont )
		self	<< cont
	end
  def self.catch_stdout( procedure )
    $stdout = output = MyIO.new #temporary copy
    procedure.call
    $stdout = STDOUT
    output
  end
end	#MyIO

#How to get stdout
puts 'vorher'
$stdout = output = MyIO.new #temporary copy
puts 'YY'
$stdout = STDOUT
puts 'nachher'
puts output


puts 'vorher'
output = MyIO.catch_stdout( proc{ puts 'xxYY' } )
puts 'nachher'
puts output